@@ -13,6 +13,7 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TypeOperators #-}
 module Basement.BoxedArray
     ( Array
     , MArray
@@ -86,6 +87,7 @@ import qualified Basement.Alg.Class as Alg
 import qualified Basement.Alg.Mutable as Alg
 import           Basement.Compat.MonadTrans
 import           Basement.Compat.Semigroup
+import           Basement.Compat.Primitive
 import           Basement.Types.OffsetSize
 import           Basement.PrimType
 import           Basement.NormalForm
@@ -131,7 +133,6 @@ instance Semigroup (Array a) where
     (<>) = append
 instance Monoid (Array a) where
     mempty  = empty
-    mappend = append
     mconcat = concat
 
 instance Show a => Show (Array a) where
